@@ -28,13 +28,15 @@
 #  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 #  POSSIBILITY OF SUCH DAMAGE.
 #
-#  $Author: haag $
+#  $Author: peter $
 #
-#  $Id: install.pl 67 2010-09-09 05:56:05Z haag $
+#  $Id: install.pl 69 2014-06-23 19:27:50Z peter $
 #
-#  $LastChangedRevision: 67 $
+#  $LastChangedRevision: 69 $
+#
+#  Last changed date:    $Date: 2014-06-23 21:27:50 +0200 (Mon, 23 Jun 2014) $
 
-require 5.6.0;
+require v5.10.1;
 
 use lib "/usr/local/libdata/perl5/site_perl/NfSen/installer-items";
 use lib "/usr/local/libdata/perl5/site_perl/NfSen";
@@ -44,6 +46,7 @@ use strict;
 use NfSen;
 use NfProfile;
 use NfSenRRD;
+use Nfsync;
 use Nfsources;
 
 # from installer-items - helper modules
@@ -52,8 +55,8 @@ use File::Copy;
 use File::Path;
 use RRDconvertv1;
 
-my $VERSION = '$Id: install.pl 67 2010-09-09 05:56:05Z haag $';
-my $nfsen_version = "1.3.5";
+my $VERSION = '$Id: install.pl 69 2014-06-23 19:27:50Z peter $';
+my $nfsen_version = "1.3.7";
 
 my @ProfileTag = ( 
 	"# \n",
@@ -101,7 +104,7 @@ sub GetPerl {
 
 	while (1) {
 		if ( -x $whichperl ) {
-			my $err = system("$whichperl -e 'require 5.6.0;'") >> 8;
+			my $err = system("$whichperl -e 'require 5.10.1;'") >> 8;
 			last if $err == 0;
 			print "Found errors while testing Perl\n";
 		} else {
@@ -200,16 +203,18 @@ sub VerifyConfig {
 	}
 
 	my @out = `$NfConf::PREFIX/nfdump -V`;
-	if ( scalar @out != 2 ) {
+	if ( scalar @out <= 0 ) {
 		die "Error getting nfdump version";
 	}
 	chomp $out[0];
-	my ($major, $minor) = $out[0] =~ /Version:\s(\d)\.(\d)\s/;
+	my ($major, $minor) = $out[0] =~ /Version:\s(\d)\.(\d+)/;
 	if ( defined $major && defined $minor) {
 		if ( $major >= 1 && $minor >= 6 ) {
 			print "Found $out[0]\n";
+		} elsif ( $out[0] =~ /Version:\s1.5.8-\d+-NSEL/ )  {
+			print "Found nfdump NSEL\n";
 		} else {
-			print "out[0]\n";
+			print "$out[0]\n";
 			die "Nfdump version not compatible with current NfSen version.\n";
 		}
 	} 
@@ -396,9 +401,9 @@ sub SetupEnv {
 	RenameFiles();
 
 	my $now = time();
-	my $tstart = $now - ( $now % 300 );
+	my $tstart = $now - ( $now % $NfConf::CYCLETIME );
 	foreach my $db ( keys %NfConf::sources ) {
-		NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/live", $db, $tstart - 300, 0);
+		NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/live", $db, $tstart - $NfConf::CYCLETIME, 0);
 	}
 	if ( $Log::ERROR ) {
 		die "Error setup RRD DBs: $Log::ERROR\n";
@@ -414,7 +419,7 @@ sub SetupEnv {
 		$profileinfo{'tbegin'}		= $tstart;
 		$profileinfo{'tstart'} 		= $tstart;
 		$profileinfo{'tend'} 		= $tstart;
-		$profileinfo{'updated'}		= $tstart - 300;
+		$profileinfo{'updated'}		= $tstart - $NfConf::CYCLETIME;
 		$profileinfo{'expire'} 		= 0;
 		$profileinfo{'maxsize'} 	= 0;
 		$profileinfo{'size'} 		= 0;
@@ -466,6 +471,7 @@ sub UpgradeProfiles {
 		return;
 	}
 
+	Nfsync::seminit();
 	$CopyRecursive::UID 	 = $nfsen_uid;
 	$CopyRecursive::GID 	 = $www_gid;
 	$CopyRecursive::MODE 	 = 0644;
@@ -484,7 +490,7 @@ sub UpgradeProfiles {
 			# estimate create time of profile
 			my $tcreate = (stat("$NfConf::PROFILESTATDIR/$profilename"))[9];
 			$profileinfo{'tcreate'}	= $tcreate;
-			$profileinfo{'tbegin'}	= $tcreate - ( $tcreate % 300 );
+			$profileinfo{'tbegin'}	= $tcreate - ( $tcreate % $NfConf::CYCLETIME );
 
 			NfProfile::WriteProfile(\%profileinfo);
 		}
@@ -547,7 +553,7 @@ sub UpgradeProfiles {
 				if( ! -f "$NfConf::PROFILEDATADIR/$profilepath/$channel/.nfstat") { 
 					# no shadow profile, but missing channel stat
 					print "Rebuilding profile stats for '$profilegroup/$profilename'\n";
-					NfProfile::DoRebuild(\%profileinfo, $profilename, $profilegroup, $profilepath, 1);
+					NfProfile::DoRebuild(\%profileinfo, $profilename, $profilegroup, $profilepath, 1, 0);
 					NfProfile::WriteProfile(\%profileinfo);
 				}
 				# make sure it's own by nfsen_uid after the rebuild
@@ -562,6 +568,7 @@ sub UpgradeProfiles {
 
 		}
 	}
+	Nfsync::semclean();
 
 
 } # End of UpgradeProfiles
@@ -577,7 +584,6 @@ sub CopyAllFiles {
 	$CopyRecursive::UID 	 = 0;
 	$CopyRecursive::GID 	 = $www_gid;
 	$CopyRecursive::MODE 	 = 0755;
-	unlink "$NfConf::BINDIR/nfsen.rc";
 	CopyRecursive::dircopy("bin", "$NfConf::INSTPREFIX$NfConf::BINDIR");
 	CopyRecursive::dircopy("libexec", "$NfConf::INSTPREFIX$NfConf::LIBEXECDIR");
 	$CopyRecursive::MODE 	 = 0644;
@@ -634,9 +640,11 @@ unshift @INC, "libexec";
 print "Check for required Perl modules: ";
 eval {
 	# other required modules
-	require RRDs; 			import RRDs;
-	require Mail::Header;	import Mail::Header;
-	require Mail::Internet;	import Mail::Internet;
+	use v5.10.1;
+	use RRDs;
+	use Mail::Header;
+	use Mail::Internet;
+	use Socket6 qw(inet_pton);
 };
 if ( $@ ) {
 	print "Failed\nRequired nfsen modules not found\n";
@@ -648,7 +656,7 @@ if ( $@ ) {
 
 # need to run as root
 if ( !NfSen::root_process() ) {
-	die "nfsen setup wants to run as root\n";
+	print "Warning: nfsen setup should normally be run as root\n";
 }
 
 # need a config file
@@ -665,7 +673,7 @@ if ( !NfConf::LoadConfig($ConfigFile) ) {
 	exit 1;
 }
 
-# check for extra errornoous nfsen.conf file, which may overwrite existing files
+# check for extra errornous nfsen.conf file, which may overwrite existing files
 if ( -f "$NfConf::CONFDIR/nfsen.conf" && -f "etc/nfsen.conf" &&
 	( (stat($ConfigFile))[1] != (stat("etc/nfsen.conf"))[1] )) {
 	die "Extra nfsen.conf file in etc directory found. Remove errornous file first"
@@ -748,7 +756,7 @@ if ( -f "$NfConf::PIDDIR/$pid_name" ) {
 			print "Stop nfsend while upgrading .";
 			kill 15, $pid;
 			my $cnt = 0;
-			while ( -f "$NfConf::PIDDIR/$pid_name" && $cnt < 300 ) {
+			while ( -f "$NfConf::PIDDIR/$pid_name" && $cnt < $NfConf::CYCLETIME ) {
 				print ".";
 				$cnt++;
 				sleep(1);
