@@ -1,4 +1,4 @@
-#
+#!/usr/bin/perl
 #  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
 #  All rights reserved.
 #
@@ -26,11 +26,11 @@
 #  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 #  POSSIBILITY OF SUCH DAMAGE.
 #
-#  $Author: haag $
+#  $Author: peter $
 #
-#  $Id: NfAlert.pm 67 2010-09-09 05:56:05Z haag $
+#  $Id: NfAlert.pm 69 2014-06-23 19:27:50Z peter $
 #
-#  $LastChangedRevision: 67 $
+#  $LastChangedRevision: 69 $
 
 package NfAlert;
 
@@ -47,6 +47,7 @@ use Mail::Internet;
 use NfSen;
 use NfSenRRD;
 use Nfcomm;
+use Nfsync;
 use Log;
 
 our %AlertPluginsCondition;
@@ -560,10 +561,10 @@ sub EvalCondition {
 	my $what_val 	= shift;
 	my $alertstatus = shift;
 
-open LOG, ">>/dev/null";
-use Data::Dumper;
-print LOG "what_val array: 0 flows, 1 packets 2 bytes\n";
-print LOG Dumper($what_val);
+## open LOG, ">>/dev/null";
+## use Data::Dumper;
+## print LOG "what_val array: 0 flows, 1 packets 2 bytes\n";
+## print LOG Dumper($what_val);
 
 	my @comp_types;
 	$comp_types[0][0] = 0;
@@ -590,8 +591,8 @@ print LOG Dumper($what_val);
 	$comp_types[2][5] = $$alertstatus{'avg12h'}{'bytes'};
 	$comp_types[2][6] = $$alertstatus{'avg24h'}{'bytes'};
 
-print LOG "comp_types array: 0 flows, 1 packets 2 bytes\n";
-print LOG Dumper(@comp_types);
+## print LOG "comp_types array: 0 flows, 1 packets 2 bytes\n";
+## print LOG Dumper(@comp_types);
 
 	# scale factors for k M G and T
 	my @scale_factor = ( 1, 1000, 1000*1000, 1000*1000*1000, 1000*1000*1000*1000 );
@@ -600,7 +601,7 @@ print LOG Dumper(@comp_types);
 	my $condition_result = 0;
 	my($op,$type,$comp,$comp_type,$stat_type,$comp_value,$scale) = split /:/, $condition;
 	
-print LOG "Process condition: $op,$type,$comp,$comp_type,$stat_type,$comp_value,$scale\n";
+## print LOG "Process condition: $op,$type,$comp,$comp_type,$stat_type,$comp_value,$scale\n";
 
 	# prepare condition
 	# this is rather complex with all variants possible
@@ -608,12 +609,12 @@ print LOG "Process condition: $op,$type,$comp,$comp_type,$stat_type,$comp_value,
 	my $a = $$what_val[$type];
 	my $b;
 	if ( $type > 2 ) {
-		# this is a per/s compare. take 300s per slot to generate average
+		# this is a per/s compare. take $NfConf::CYCLETIME per slot to generate average
 		if ( $type == 5 ) {
 			# convert bytes to bits
-			$b = 8 * $comp_types[$type-3][$comp_type]/300;
+			$b = 8 * $comp_types[$type-3][$comp_type]/$NfConf::CYCLETIME;
 		} else {
-			$b = $comp_types[$type-3][$comp_type]/300;
+			$b = $comp_types[$type-3][$comp_type]/$NfConf::CYCLETIME;
 		}
 	} else {
 		# this is an absolute compare
@@ -625,23 +626,23 @@ print LOG "Process condition: $op,$type,$comp,$comp_type,$stat_type,$comp_value,
 	} else {	# absolute range
 		$value = $comp_value * $scale_factor[$scale];
 	}
-print LOG "a: $a, b: $b value: $value\n";
+## print LOG "a: $a, b: $b value: $value\n";
 	$condition_result = 0;
 	if ( $comp == 2 ) { # outside
-print LOG "comp type = 2 'outside'\n";
+## print LOG "comp type = 2 'outside'\n";
 		my $b1 = $b + $value;
 		my $b2 = $b > $value ? $b - $value : 0;
 		$condition_result = ($a > $b1) || ($a < $b2);
-print LOG "($a > $b1) || ($a < $b2) = $condition_result\n";
+## print LOG "($a > $b1) || ($a < $b2) = $condition_result\n";
 	} else {
 		# comp '<' or '>'
 		if ( $comp == 0 ) {	# '>'
 			$b += $value;
-print LOG "comp type 0 '>'\n";
+## print LOG "comp type 0 '>'\n";
 			$condition_result = $a > $b;
-print LOG "$a > $b = $condition_result\n";
+## print LOG "$a > $b = $condition_result\n";
 		} elsif ( $comp == 1 ) { # '<'
-print LOG "comp type 1 '<'\n";
+## print LOG "comp type 1 '<'\n";
 			if ( $comp_type == 0 ) {
 				$b = $value;
 			} else {
@@ -649,15 +650,15 @@ print LOG "comp type 1 '<'\n";
 				$b = $b > $value ? $b - $value : 0;
 			}
 			$condition_result = $a < $b;
-print LOG "$a < $b = $condition_result\n";
+## print LOG "$a < $b = $condition_result\n";
 		}
 	}
-close LOG;
+## close LOG;
 
 	$condition_result = $condition_result ? 1 : 0;
 	return($condition_result, $op);
 
-} # Ebnd of EvalCondition
+} # End of EvalCondition
 
 sub EvalStack {
 	my $val_list = shift;
@@ -733,7 +734,7 @@ sub GetTop1Stat {
        		}
    		}
 	};
-print "ARG: $stat\n";
+## print "ARG: $stat\n";
 	my @output;
 	if ( open NFDUMP, "$NfConf::PREFIX/nfdump -r $file -o pipe -q -n 1 $stat 2>&1|" ) {
 		local $SIG{PIPE} = sub { syslog('err', "Pipe broke for nfprofile"); };
@@ -1019,11 +1020,11 @@ sub RunPeriodic {
 					$what_val[1] = $$alertstatus{'last'}{'packets'}->[0];
 					$what_val[2] = $$alertstatus{'last'}{'bytes'}->[0];
 					# update per/s statistic
-					# take 300s. this may vary from actual duration, which needs not to be exactly 300s
-					# but for now stay with 300s
-					$what_val[3] = $$alertstatus{'last'}{'flows'}->[0] / 300;
-					$what_val[4] = $$alertstatus{'last'}{'packets'}->[0] / 300;
-					$what_val[5] = 8 * $$alertstatus{'last'}{'bytes'}->[0] / 300;
+					# take $NfConf::CYCLETIME. this may vary from actual duration, which needs not to be exactly $NfConf::CYCLETIME
+					# but for now stay with $NfConf::CYCLETIME
+					$what_val[3] = $$alertstatus{'last'}{'flows'}->[0] / $NfConf::CYCLETIME;
+					$what_val[4] = $$alertstatus{'last'}{'packets'}->[0] / $NfConf::CYCLETIME;
+					$what_val[5] = 8 * $$alertstatus{'last'}{'bytes'}->[0] / $NfConf::CYCLETIME;
 					#if ( $$alertstatus{'duration'}[0] != 0 ) {
 						#$what_val[3] = $$alertstatus{'last'}{'flows'}->[0] / ($$alertstatus{'duration'}[0] / 1000);
 						#$what_val[4] = $$alertstatus{'last'}{'packets'}->[0] / ($$alertstatus{'duration'}[0] / 1000);
@@ -1366,9 +1367,9 @@ sub AddAlert {
 		$action_email =~ s/^\s+//;
 		$action_email =~ s/\s$//;
 		foreach my $email_addr ( split /\s*,\s*/, $action_email ) {
-			if ( $action_email !~ /^([A-Z0-9]+[._]?){1,}[A-Z0-9\-]+\@(([A-Z0-9]+[-]?){1,}[A-Z0-9]+\.){1,}[A-Z]{2,4}$/i ) {
+			if ( !NfSen::ValidEmail($email_addr)) {
 				print $socket $EODATA;
-				print $socket "ERR action_email '$action_email' not a valid email address\n";
+				print $socket "ERR action_email '$email_addr' not a valid email address\n";
 				return;
 			}
 		}
@@ -1607,7 +1608,7 @@ sub AddAlert {
 	
 	InitRunInfo($alert, $status eq 'enabled' ? 1 : 0);
 	foreach my $type ( 'flows', 'packets', 'bytes' ) {
-		NfSenRRD::SetupAlertRRD("$NfConf::PROFILESTATDIR/~$alert/", "avg-$type", $liveprofile{'tend'} - 300, \@DSlist);
+		NfSenRRD::SetupAlertRRD("$NfConf::PROFILESTATDIR/~$alert/", "avg-$type", $liveprofile{'tend'} - $NfConf::CYCLETIME, \@DSlist);
     	if ( defined $Log::ERROR ) {
 			print $socket $EODATA;
         	print $socket "ERR Creating alert RRD for '$alert' failed: $Log::ERROR\n";
@@ -1747,7 +1748,7 @@ sub ModifyAlert {
 		$action_email =~ s/^\s+//;
 		$action_email =~ s/\s$//;
 		foreach my $email_addr ( split /\s*,\s*/, $action_email ) {
-			if ( $email_addr !~ /^([A-Z0-9]+[._]?){1,}[A-Z0-9]+\@(([A-Z0-9]+[-]?){1,}[A-Z0-9]+\.){1,}[A-Z]{2,4}$/i ) {
+			if ( !NfSen::ValidEmail($email_addr) ) {
 				print $socket $EODATA;
 				print $socket "ERR action_email '$email_addr' not a valid email address\n";
 				return;
@@ -1755,7 +1756,7 @@ sub ModifyAlert {
 		}
 		$alertinfo{'action_email'} = $action_email;
 		if ( $alertinfo{'action_subject'} eq "" ) {
-			$alertinfo{'action_subject'} = "ALert triggered";
+			$alertinfo{'action_subject'} = "Alert triggered";
 		}
 	}
 
@@ -1961,19 +1962,19 @@ sub DeleteAlert {
 		push @dirs, "$NfConf::PROFILEDATADIR";
 	}
 	foreach my $dir ( @dirs ) {
-		if ( !main::semnowait($main::semlock) ) {
+		if ( !Nfsync::semnowait() ) {
 			print $socket $EODATA;
 			print $socket "ERR Can not delete the alert while a periodic update is in progress. Try again later.\n";
 			return;
 		}
 
 		if ( !rename "$dir/~$alert", "$dir/.~$alert" ) {
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 			print $socket $EODATA;
 			print $socket "ERR Failed to rename alert '$alert' in order to delete: $!\n";
 			return;
 		} else {
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 		}
 
 		my $command = "/bin/rm -rf $dir/.~$alert &";
