@@ -1,4 +1,4 @@
-#
+#!/usr/bin/perl
 #  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
 #  All rights reserved.
 #
@@ -26,11 +26,11 @@
 #  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 #  POSSIBILITY OF SUCH DAMAGE.
 #
-#  $Author: haag $
+#  $Author: peter $
 #
-#  $Id: NfProfile.pm 67 2010-09-09 05:56:05Z haag $
+#  $Id: NfProfile.pm 69 2014-06-23 19:27:50Z peter $
 #
-#  $LastChangedRevision: 67 $
+#  $LastChangedRevision: 69 $
 
 package NfProfile;
 
@@ -42,6 +42,7 @@ use Fcntl qw(:DEFAULT :flock);
 
 use NfSen;
 use NfSenRRD;
+use Nfsync;
 use Log;
 
 our $PROFILE_VERSION = 130;	# version 1.3.0
@@ -483,7 +484,7 @@ sub ReadRRDStatInfo {
 	my $profilepath = ProfilePath($name, $profilegroup);
 
 	my $RRDdb = "$NfConf::PROFILESTATDIR/$profilepath/$channel.rrd";
-	my ($start,$step,$names,$data) = RRDs::fetch $RRDdb, "-s", $tstart, "-e", $tend, "MAX";
+	my ($start,$step,$names,$data) = RRDs::fetch $RRDdb, "-s", $tstart-300, "-e", $tend-300, "MAX";
 	my $err=RRDs::error;
 	if ( defined $err ) {
 		return ($statinfo, $err);
@@ -493,7 +494,7 @@ sub ReadRRDStatInfo {
 		my $i = 0;
 		foreach my $val (@$line) {
 			if ( defined $val ) {
-				$$statinfo{$$names[$i++]} += int(300 * $val);
+				$$statinfo{$$names[$i++]} += int($NfConf::CYCLETIME * $val);
 			}
 		}
 	}
@@ -549,7 +550,7 @@ sub GetPeakValues {
 	}
 
 	my $err = undef;
-	for ( my $t=$tmin; $t<=$tmax; $t += 300 ) {
+	for ( my $t=$tmin; $t<=$tmax; $t += $NfConf::CYCLETIME ) {
 		$sum_pos = 0;
 		$sum_neg = 0;
 		my $subdirs = NfSen::SubdirHierarchy($t);
@@ -897,7 +898,7 @@ sub ProfileHistory {
 	}
 
 	# we have to process that many time slices:
-	my $numslices = ((( $tend - $tstart ) / 300 ) + 1 );
+	my $numslices = ((( $tend - $tstart ) / $NfConf::CYCLETIME ) + 1 );
 	
 	$$profileref{'status'} = 'built 0';
 	$$profileref{'locked'} = 1;
@@ -914,7 +915,8 @@ sub ProfileHistory {
 
 	my $profilepath = ProfilePath($name, $group);
 	my $subdirlayout = $NfConf::SUBDIRLAYOUT ? "-S $NfConf::SUBDIRLAYOUT" : "";
-	my $arg = "-I -p $NfConf::PROFILEDATADIR -P $NfConf::PROFILESTATDIR $subdirlayout $NfConf::ZIPprofiles";
+	my $arg = "-I -p $NfConf::PROFILEDATADIR -P $NfConf::PROFILESTATDIR $subdirlayout ";
+	$arg   .= "-z " if $NfConf::ZIPprofiles;
 
 	# create argument list specific for each channel
 	# at the moment this contains of all channels in a continues profile
@@ -955,12 +957,12 @@ sub ProfileHistory {
 		}
 
 		# run nfprofile but only for new profile $name
-		syslog('debug', "Run profiler: '$arg' '$flist'");
+		#syslog('debug', "Run profiler: '$arg' '$flist'");
 		if ( open NFPROFILE, "| $NfConf::PREFIX/nfprofile -t $t $arg $flist" ) {
 			local $SIG{PIPE} = sub { syslog('err', "Pipe broke for nfprofile"); };
 			foreach my $profileopts ( @ProfileOptList ) {
 				print NFPROFILE "$profileopts\n";
-				syslog('debug', "profile opts: $profileopts");
+				#syslog('debug', "profile opts: $profileopts");
 			}
 			close NFPROFILE;	# SIGCHLD sets $child_exit
 		} 
@@ -979,7 +981,7 @@ sub ProfileHistory {
 				$profile_size +=(stat($outfile))[7];
 			}
 		}
-		$t += 300;
+		$t += $NfConf::CYCLETIME;
 		if ( $continous_profile && $t == $tend ) {
 			my %liveprofile = ReadProfile('live', '.');
 			$tend = $liveprofile{'tend'};
@@ -1012,7 +1014,7 @@ sub ProfileHistory {
 	}
 
 	$$profileref{'size'} 	= $profile_size;
-	$$profileref{'updated'} = $t;
+	$$profileref{'updated'} = $tend;
 
 	return $profile_size;
 
@@ -1085,8 +1087,8 @@ sub AddChannel {
 	$$profileref{'channel'}{$channel}{'sourcelist'} = $sourcelist;
 
 	# $tstart is the first value we need in the RRD DB, therefore specify 
-	# $tstart - 300 ( 1 slot )
-	NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/$profilepath", $channel, $tstart - 300, 1);
+	# $tstart - $NfConf::CYCLETIME ( 1 slot )
+	NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/$profilepath", $channel, $tstart - $NfConf::CYCLETIME, 1);
 	if ( defined $Log::ERROR ) {
 		rmdir "$NfConf::PROFILEDATADIR/$profilepath/$channel",
 		unlink $filter;
@@ -1185,11 +1187,21 @@ sub GetProfilegroups {
 } # End of GetProfilegroups
 
 sub DoRebuild {
+	my $socket		 = shift;
 	my $profileinfo  = shift;
 	my $profile 	 = shift;
 	my $profilegroup = shift;
 	my $profilepath  = shift;
 	my $installer	 = shift;
+	my $DoGraphs	 = shift;
+
+	# rebuilding live is a bit trickier to keep everything consistent
+	# make sure, the periodic update process is done - then block cycles
+	syslog('info', "Rebuilding profile '$profile', group '$profilegroup'");
+	if ( $profile eq 'live' && $profilegroup eq '.' ) {
+		syslog('debug', "Get lock");
+		Nfsync::semwait();
+	}
 
 	# rebuild each channel, using nfexpire
 	my $profilesize = 0;
@@ -1281,6 +1293,149 @@ sub DoRebuild {
 		$order++;
 	}
 
+	# in case of rebuilding the graphs
+	if ( $DoGraphs ) {
+		syslog('info', "Rebuilding graphs");
+		$$profileinfo{'status'} = 'rebuilding';
+		$$profileinfo{'locked'} = 1;
+		if ( !WriteProfile($profileinfo) ) {
+			syslog('err', "Error writing profile '$profile': $Log::ERROR");
+			return "Failed writing profile";
+		}
+
+		my $continous_profile = ($$profileinfo{'type'} & 3) == 2;
+
+		syslog('info', "Setting up RRD DBs");
+		foreach my $channel ( @CHAN ) {
+			NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/$profilepath", $channel, $tstart - $NfConf::CYCLETIME, 1);
+		}
+
+		my $numslices = ((( $tend - $tstart ) / $NfConf::CYCLETIME ) + 1 );
+		my $percent  = $numslices / 100;
+		my $counter  = 0;
+		my $progress = 0;
+		my $modulo	 = int ($percent * 10) < 2 ? 2 : int ($percent * 10);
+
+		my $t;
+		for ( $t = $tstart; $t <= $tend; $t += $NfConf::CYCLETIME ) {
+			my $t_iso 	= NfSen::UNIX2ISO($t);
+			foreach my $channel ( @CHAN ) {
+				my $subdirs = NfSen::SubdirHierarchy($t);
+				my ($statinfo, $exit_code, $err ) = ReadStatInfo($profileinfo, $channel, $subdirs, $t_iso, undef);
+				my @_values = ();
+				foreach my $ds ( @NfSenRRD::RRD_DS ) {
+           			if ( !defined $$statinfo{$ds} || $$statinfo{$ds} == - 1 ) {
+              			push @_values, 0;
+          			} else {
+               			push @_values, $$statinfo{$ds};
+           			}
+				}
+				$err = NfSenRRD::UpdateDB("$NfConf::PROFILESTATDIR/$profilepath", $channel, $t,
+						join(':',@NfSenRRD::RRD_DS) , join(':', @_values));
+				if ( $Log::ERROR ) {
+					syslog('err', "ERROR Update RRD time: '$t_iso', db: '$channel', profile: '$profile' group '$profilegroup' : $Log::ERROR");
+				}
+			}
+
+			$counter++;
+			my $completed = sprintf "%.1f%%", $counter / $percent;
+			if ( $completed > 100 ) {
+				$completed = 100;
+			}
+			if ( ($counter % $modulo ) == 0 ) {
+				print $socket ".info Rebuilding Profile '$profile': Completed: $completed\%\n";
+				syslog('info', "Rebuilding Profile '$profile': Completed: $completed\%");
+			}
+
+
+		}
+
+		# history data is updated and we are done with history profiles
+		# A continous profile except 'live' must be updated up to the current slot 
+		# rebuilding the profile could have missed new slots -> profile missing slots
+		if ( $continous_profile && ($profile ne 'live' || $profilegroup ne '.') ) {
+			# get current current time slot
+			my %liveprofile = ReadProfile('live', '.');
+			$tend = $liveprofile{'tend'};
+
+			# this can happen, if a history profile is switch back to continous
+			# profile and wants to get updated now. Be sure we have all data
+			if ( $t < $liveprofile{'tstart'} ) {
+				$t = $liveprofile{'tstart'};
+			}
+			my $profile_size = $$profileinfo{'size'};
+
+			# prepare profiler args 
+			my @ProfileOptList;
+			foreach my $channel ( keys %{$$profileinfo{'channel'}} ) {
+				push @ProfileOptList, "$profilegroup#$profile#$$profileinfo{'type'}#$channel#$$profileinfo{'channel'}{$channel}{'sourcelist'}";
+			}
+			my $channellist = join ':', keys %{$liveprofile{'channel'}};
+			my $subdirlayout = $NfConf::SUBDIRLAYOUT ? "-S $NfConf::SUBDIRLAYOUT" : "";
+			my $arg = "-I -p $NfConf::PROFILEDATADIR -P $NfConf::PROFILESTATDIR $subdirlayout ";
+			$arg   .= "-z " if $NfConf::ZIPprofiles;
+
+			# profile missing slots
+			if ( $t <= $tend ) {
+				syslog('info', "Profiling missing slots");
+			}
+			while ( $t <= $tend ) {
+				if ( -f "$NfConf::PROFILESTATDIR/$profilepath/.CANCELED" ) {
+					$$profileinfo{'status'} = 'canceled';
+					syslog('info', "Rebuild canceled.");
+					last;
+				}
+
+				my $iso = NfSen::UNIX2ISO($t);
+				my $subdirs = NfSen::SubdirHierarchy($t);
+				my %statinfo	= ();
+				my $dsvector = join(':', @NfSenRRD::RRD_DS);
+		
+				my $flist = "-M $NfConf::PROFILEDATADIR/live/$channellist -r nfcapd.$iso";
+		
+				# run nfprofile for remaining slots
+				#syslog('debug', "Run profiler: '$arg' '$flist'");
+				if ( open NFPROFILE, "| $NfConf::PREFIX/nfprofile -t $t $arg $flist" ) {
+					local $SIG{PIPE} = sub { syslog('err', "Pipe broke for nfprofile"); };
+					foreach my $profileopts ( @ProfileOptList ) {
+						print NFPROFILE "$profileopts\n";
+						#syslog('debug', "profile opts: $profileopts");
+					}
+					close NFPROFILE;	# SIGCHLD sets $child_exit
+				} 
+			
+				if ( $main::child_exit != 0 ) {
+					syslog('err', "nfprofile failed: $!\n");
+					syslog('debug', "System was: $NfConf::PREFIX/nfprofile $arg $flist");
+					next;
+				} 
+		
+				foreach my $channel ( keys %{$$profileinfo{'channel'}} ) {
+					my $outfile = "$NfConf::PROFILEDATADIR/$profilepath/$channel/$subdirs/nfcapd.$iso";
+		
+					# array element 7 contains the size in bytes
+					$profile_size +=(stat($outfile))[7];
+				}
+				$$profileinfo{'updated'} = $t;
+				$$profileinfo{'tend'} 	 = $t;
+				$$profileinfo{'size'} 	 = $profile_size;
+
+				$t += $NfConf::CYCLETIME;
+				%liveprofile = ReadProfile('live', '.');
+				$tend = $liveprofile{'tend'};
+			}
+		}
+	} else {
+		syslog('info', "Graphs for profile '$profile', group '$profilegroup' not rebuilded");
+	}
+
+	if ( $profile eq 'live' && $profilegroup eq '.' ) {
+		syslog('debug', "Release lock");
+		Nfsync::semsignal();
+	}
+
+	syslog('info', "Rebuilding done.");
+
 	# if we need to create the profile from
 	# history data, lock the profile until down.
 	$$profileinfo{'locked'} 		= 0;
@@ -1681,8 +1836,8 @@ sub AddProfile {
 	$profileinfo{'tstart'} 		= $tstart;
 	$profileinfo{'tend'} 		= $tend;
 
-	# the first slot to be updated is 'updated' + 300
-	$profileinfo{'updated'}		= $tstart - 300;	
+	# the first slot to be updated is 'updated' + $NfConf::CYCLETIME
+	$profileinfo{'updated'}		= $tstart - $NfConf::CYCLETIME;	
 
 	# expiring profiles makes only sense for continous profiles
 	$profileinfo{'expire'} 		= $continuous_profile ? $lifetime : 0;
@@ -1960,9 +2115,9 @@ sub DeleteProfileChannel {
 		}
 	}
 
-	if ( main::semnowait($main::semlock) ) {
+	if ( Nfsync::semnowait() ) {
 		$ret = NfProfile::DeleteChannel(\%profileinfo, $channel);
-		main::semsignal($main::semlock);
+		Nfsync::semsignal();
 	} else {
 		print $socket $EODATA;
 		print $socket "ERR Can not delete the channel while a periodic update is in progress. Try again later.\n";
@@ -2088,7 +2243,7 @@ sub DeleteProfile {
 		push @dirs, "$NfConf::PROFILEDATADIR";
 	}
 	foreach my $dir ( @dirs ) {
-		if ( !main::semnowait($main::semlock) ) {
+		if ( !Nfsync::semnowait() ) {
 			open DELFLAG, ">$NfConf::PROFILESTATDIR/$profilepath/.DELETED";
 			print DELFLAG "profile deleted and to be removed later\n";
 			close DELFLAG;
@@ -2098,12 +2253,12 @@ sub DeleteProfile {
 		}
 
 		if ( !rename "$dir/$profilepath", "$dir/.$profile" ) {
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 			print $socket $EODATA;
 			print $socket "ERR Failed to rename profile '$profile' in group '$profilegroup' in order to delete: $!\n";
 			return;
 		} else {
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 		}
 
 		my $command = "/bin/rm -rf $dir/.$profile &";
@@ -2144,7 +2299,7 @@ sub ReGroupProfile {
 		return "An other profile with name '$profile' already exists in new group\n";
 	}
 
-	if ( !main::semnowait($main::semlock) ) {
+	if ( !Nfsync::semnowait() ) {
 		return "Can not rename the profile while a periodic update is in progress. Try again later.\n";
 	}
 
@@ -2158,21 +2313,21 @@ sub ReGroupProfile {
 		if ( ! -d "$dir/$newgroup" && !mkdir "$dir/$newgroup" ) {
 			my $err = "Can't create new profile group directory '$NfConf::PROFILESTATDIR/$newgroup': $!";
 			syslog("err", "$err");
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 			return "$err\n";
 		}
 	
 		if ( !open TAGFILE, ">$dir/$newgroup/.group" ) {
 			my $err = $!;
 			syslog("err", "Can't create profile group tag file '$dir/$newgroup/.group': $err");
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 			return "Can't create profile group tag file '$dir/$newgroup/.group': $err!\n";
 		}
 		close TAGFILE;
 	}
 
 	if ( !rename "$NfConf::PROFILESTATDIR/$profilepath", "$NfConf::PROFILESTATDIR/$newprofilepath" ) {
-		main::semsignal($main::semlock);
+		Nfsync::semsignal();
 		return "Failed to rename profile '$profile': $!\n";
 	} 
 	if ( "$NfConf::PROFILESTATDIR" ne "$NfConf::PROFILEDATADIR" ) {
@@ -2180,13 +2335,13 @@ sub ReGroupProfile {
 			# ohhh this is really bad! try to restore old profile stat dir
 			rename "$NfConf::PROFILESTATDIR/$newprofilepath", "$NfConf::PROFILESTATDIR/$profilegroup";
 
-			main::semsignal($main::semlock);
+			Nfsync::semsignal();
 			return "Failed to rename profile '$profile': $!\n";
 		} 
 	}
 
 	$$profileref{'group'} = $newgroup;
-	main::semsignal($main::semlock);
+	Nfsync::semsignal();
 
 	return 'ok';
 
@@ -2236,7 +2391,7 @@ sub CommitProfile {
 	}
 
 	my $now	= time();
-	$now -= $now % 300;
+	$now -= $now % $NfConf::CYCLETIME;
 
 	if ( $profileinfo{'status'} eq 'stalled' ) {
 		$profileinfo{'tstart'} 	= $now;
@@ -2488,7 +2643,7 @@ print $socket ".Adjust cont profile tstart to live profile tstart $liveprofile{'
 print $socket ".New type is real\n";	
 				if ( $new_type == 2 ) {
 					my $now = time();
-					$profileinfo{'tstart'} = $now - ( $now % 300 );
+					$profileinfo{'tstart'} = $now - ( $now % $NfConf::CYCLETIME );
 				} else {
 					print $socket $EODATA;
 					print $socket "ERR Can not convert to history profile - no data available\n";	
@@ -2785,9 +2940,11 @@ sub RebuildProfile {
 		return;
 	}
 
+	my $RebuildGraphs = exists $$opts{'all'} ? 1 : 0;
+		
 	syslog('info', "Start to rebuild profile '$profile'");
 
-	my $status = DoRebuild(\%profileinfo, $profile, $profilegroup, $profilepath, 0);
+	my $status = DoRebuild($socket, \%profileinfo, $profile, $profilegroup, $profilepath, 0, $RebuildGraphs);
 
 	if ( !WriteProfile(\%profileinfo) ) {
 		syslog('err', "Error writing profile '$profile': $Log::ERROR");
@@ -3284,7 +3441,7 @@ sub CheckProfiles {
 				} else {
 					$profileinfo{'type'} = 1;
 				}
-				my $status = DoRebuild(\%profileinfo, $profile, $profilegroup, $profilepath, 0);
+				my $status = DoRebuild(\%profileinfo, $profile, $profilegroup, $profilepath, 0, 0);
 				syslog('err', "Rebuilded profile '$profile' in group '$profilegroup': $status ");
 			}
 			if ( -f "$NfConf::PROFILESTATDIR/$profilepath/.CANCELED" ) {
@@ -3296,7 +3453,7 @@ sub CheckProfiles {
 				} else {
 					$profileinfo{'type'} = 1;
 				}
-				my $status = DoRebuild(\%profileinfo, $profile, $profilegroup, $profilepath, 0);
+				my $status = DoRebuild(\%profileinfo, $profile, $profilegroup, $profilepath, 0, 0);
 				syslog('err', "Rebuilded profile '$profile' in group '$profilegroup': $status ");
 			}
 			if ( $profileinfo{'locked'} ) {
