@@ -27,11 +27,11 @@
 #  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 #  POSSIBILITY OF SUCH DAMAGE.
 #
-#  $Author: haag $
+#  $Author: peter $
 #
-#  $Id: Nfcomm.pm 67 2010-09-09 05:56:05Z haag $
+#  $Id: Nfcomm.pm 69 2014-06-23 19:27:50Z peter $
 #
-#  $LastChangedRevision: 67 $
+#  $LastChangedRevision: 69 $
 package Nfcomm;
 
 use strict;
@@ -45,7 +45,8 @@ use POSIX 'strftime';
 use POSIX 'setsid';
 use POSIX ":sys_wait_h";
 use Sys::Syslog;
-use UNIVERSAL qw( can );
+use Nfsync;
+use UNIVERSAL;
 
 use Log;
 use NfProfile;
@@ -166,14 +167,11 @@ sub GetGlobals {
 	my ($offhour, $offmin) = strftime('%z', localtime) =~ /^(.\d{2})(\d{2})/;
 	$GMToffset 		= $offhour * 3600 + $offmin * 60;
 
-#	print $socket "\$DOCURL	 = \"/nfsen/doc\";\n";
-	print $socket "_globals=\$PROFILEDATADIR = \"$NfConf::PROFILEDATADIR\";\n";
 	print $socket "_globals=\$SUBDIRLAYOUT = \"$NfConf::SUBDIRLAYOUT\";\n";
 	print $socket "_globals=\$FRONTEND_PLUGINDIR = \"$NfConf::FRONTEND_PLUGINDIR\";\n";
 	print $socket "_globals=\$RRDoffset = \"$NfConf::RRDoffset\";\n";
-#	print $socket "_globals=\$DEBUG = \"$NfConf::DEBUG\";\n";
-#	print $socket "\$LIBEXECDIR = \"$NfConf::LIBEXECDIR\";\n";
 
+	print $socket "_globals=\$CYCLETIME = \"$NfConf::CYCLETIME\";\n";
 	print $socket "_globals=\$Refresh = \"$NfConf::Refresh\";\n";
 	print $socket "_globals=\$GMToffset = $GMToffset;\n";
 	print $socket "_globals=\$TZname = \"$TZname\";\n";
@@ -189,10 +187,10 @@ sub SemCheck {
 	my $socket  = shift;
 	my $opts	= shift;
 
-	print $socket ". Semaphore $main::semlock\n";
+	print $socket ". Semaphore Nfsync::semlock\n";
 	print $socket $EODATA;
-	if ( main::semnowait($main::semlock) ) {
-		main::semsignal($main::semlock);
+	if ( Nfsync::semnowait() ) {
+		Nfsync::semsignal();
 		print $socket "OK Semaphore available\n";
 	} else {
 		print $socket "ERR Semaphore not available\n";
@@ -802,6 +800,7 @@ sub Setup_Server {
 			close $server;
 			return undef;
 		}
+		chown $NfConf::UID, $NfConf::GID, $socket_path; 
 		chmod 0660, $socket_path;
 
 	} else {
@@ -858,7 +857,7 @@ sub spawn {
 		return; # I'm the parent
 	}
 	# else I'm the child -- go spawn
-print STDERR "Spawn new process\n";
+
 	open(STDIN,  "<&Client")   || die "can't dup client to stdin";
 	open(STDOUT, ">&Client")   || die "can't dup client to stdout";
 	## open(STDERR, ">&STDOUT") || die "can't dup stdout to stderr";
@@ -1093,7 +1092,9 @@ sub RunServer {
 	my $timeout = 10;
 	my $silent  = 0;
 	my $alarm = 0;
+	my $SIGpipe = 0;
 	$SIG{ALRM} = sub { $alarm = 1; };
+	$SIG{PIPE} = sub { $SIGpipe = 1 };
 	while ( !$done ) {
 		$paddr = accept(Client,$server);
 		if ( !$paddr ) {
@@ -1130,6 +1131,7 @@ sub RunServer {
 				eval {
 					local $SIG{'__DIE__'} = 'DEFAULT';
 					local $SIG{ALRM} = sub { die "Timeout reading from socket!"; };
+					local $SIG{PIPE} = sub { die "Broken PIPE while reading"; };
 					my $done = 0;
 				
 					# read multi line command and args
@@ -1223,7 +1225,8 @@ sub nfsend_connect {
 
 	eval {
 		local $SIG{'__DIE__'} = 'DEFAULT';
-		local $SIG{ALRM} = sub { die; };
+		local $SIG{ALRM} = sub { die "Timeout nfsend_connect"; };
+		local $SIG{PIPE} = sub { die "Signal PIPE nfsend_connect"; };
 
 		alarm 30;
 		my $greeting = <$nfsen_sock>;
@@ -1311,7 +1314,8 @@ sub nfsend_comm {
 	my $status = '';
 	eval {
 		local $SIG{'__DIE__'} = 'DEFAULT';
-		local $SIG{ALRM} = sub { die; };
+		local $SIG{ALRM} = sub { die "Timeout on socket"; };
+		local $SIG{PIPE} = sub { die "Broken Pipe"; };
 
 		# send command and options
 # print "Send: $command\n";
@@ -1336,6 +1340,11 @@ sub nfsend_comm {
 			# Debug output from nfsend
 			if ( $line =~ /^\..+/ ) {
 				print "DEBUG: $line" if $PRINT_DEBUG;
+				if ( defined $comm_opts && defined $$comm_opts{'info'} && $line =~ /^\.info /) {
+					$line =~ s/\.info //;	# cut header
+					$line =~ s/\n//;		# cut potential EOL
+					print "Info: $line\n";
+				}
 				next;
 			}
 
